@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.34 2004/08/25 08:45:33 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.35 2004/09/13 00:05:10 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -38,6 +38,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/time.h>
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -68,6 +69,8 @@ void do_child(int unix_fd, int inet_fd)
 	StartupPacket *sp;
 	POOL_CONNECTION *frontend;
 	POOL_CONNECTION_POOL *backend;
+	struct timeval now;
+	struct timezone tz;
 
 	pool_debug("I am %d", getpid());
 
@@ -88,6 +91,10 @@ void do_child(int unix_fd, int inet_fd)
 	}
 #endif
 
+	/* initialize random seed */
+	gettimeofday(&now, &tz);
+	srandom((unsigned int) now.tv_usec);
+
 	/* initialize connection pool */
 	if (pool_init_cp())
 	{
