@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.18 2004/09/13 00:05:56 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.19 2004/09/30 12:34:11 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -24,6 +24,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
+#include <netdb.h>
+#include <arpa/inet.h>
 
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -50,7 +52,7 @@ static int read_pid_file(void);
 static void write_pid_file(void);
 static pid_t fork_a_child(int unix_fd, int inet_fd);
 static int create_unix_domain_socket(void);
-static int create_inet_domain_socket(void);
+static int create_inet_domain_socket(const char *hostname);
 static void myexit(int code);
 
 static RETSIGTYPE exit_handler(int sig);
@@ -181,9 +183,9 @@ int main(int argc, char **argv)
 	unix_fd = create_unix_domain_socket();
 
 	/* create inet domain socket if any */
-	if (pool_config.inetdomain)
+	if (pool_config.listen_addresses[0])
 	{
-		inet_fd = create_inet_domain_socket();
+		inet_fd = create_inet_domain_socket(pool_config.listen_addresses);
 	}
 
 	pids = malloc(pool_config.num_init_children * sizeof(pool_config.num_init_children));
@@ -371,7 +373,7 @@ pid_t fork_a_child(int unix_fd, int inet_fd)
 /*
 * create inet domain socket
 */
-static int create_inet_domain_socket(void)
+static int create_inet_domain_socket(const char *hostname)
 {
 	struct sockaddr_in addr;
 	int fd;
@@ -391,9 +393,27 @@ static int create_inet_domain_socket(void)
 		pool_error("setsockopt() failed. reason: %s", strerror(errno));
 		myexit(1);
 	}
+
 	memset((char *) &addr, 0, sizeof(addr));
 	((struct sockaddr *)&addr)->sa_family = AF_INET;
-	addr.sin_addr.s_addr = htonl(INADDR_ANY);
+
+	if (strcmp(hostname, "*")==0)
+	{
+		addr.sin_addr.s_addr = htonl(INADDR_ANY);
+	}
+	else
+	{
+		struct hostent *hostinfo;
+
+		hostinfo = gethostbyname(hostname);
+		if (!hostinfo)
+		{
+			pool_error("could not resolve host name \"%s\": %s", hostname, hstrerror(h_errno));
+			myexit(1);
+		}
+		addr.sin_addr = *(struct in_addr *) hostinfo->h_addr;
+	}
+
 	addr.sin_port = htons(pool_config.port);
 	len = sizeof(struct sockaddr_in);
 	status = bind(fd, (struct sockaddr *)&addr, len);
