@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.30 2004/09/13 00:05:10 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.32 2004/09/30 12:34:11 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -283,7 +283,24 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 			}
 			if (kind1 == 0)
 			{
-				pool_read(SECONDARY(backend), &kind1, 1);
+				/* at this point the query should have completed and it's safe to set timeout here */
+				pool_debug("pool_process_query: waiting for secondary for data ready");
+
+				/* temporary enable timeout */
+				pool_enable_timeout();
+
+				if (pool_check_fd(SECONDARY(backend), 0))
+				{
+					pool_error("pool_process_query: secondary data is not ready at synchronous point. abort this session");
+
+				}
+				else
+				{
+					pool_read(SECONDARY(backend), &kind1, 1);
+				}
+
+				pool_disable_timeout();
+
 			}
 			if (kind == '\0' || kind != kind1)
 			{
@@ -492,6 +509,11 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		{
 			return POOL_END;
 		}
+
+		/* in "strict mode" we need to wait for secondary completing the query */
+		if (pool_config.replication_strict || STRICT_MODE(string))
+			if (synchronize(SECONDARY(backend)))
+				return POOL_END;
 	}
 	return POOL_CONTINUE;
 }
@@ -1462,9 +1484,9 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 
 	i = 0;
 
-	status[i].name = "inetdomain";
-	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.inetdomain);
-	status[i].desc = "1 if accepting TCP/IP connection";
+	status[i].name = "listen_addresses";
+	snprintf(status[i].value, MAXVALLEN, "%s", pool_config.listen_addresses);
+	status[i].desc = "host name(s) or IP address(es) to listen to";
 	i++;
 
 	status[i].name = "port";
