@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
-* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.21 2004/07/19 01:28:44 t-ishii Exp $
+* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.23 2004/09/02 02:58:40 t-ishii Exp $
 *
 * pgpool: a language independent connection pool server for PostgreSQL 
 * written by Tatsuo Ishii
@@ -108,20 +108,29 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 
 	int consume_size;
 	int readlen;
+	int notimeout;
 
 	consume_size = consume_pending_data(cp, buf, len);
 	len -= consume_size;
 	buf += consume_size;
 
+	/* if this is the secondary backend, then set timeout */
+	notimeout = !cp->issecondary_backend;
+
 	while (len > 0)
 	{
-		if (cp->issecondary_backend)
+		if (pool_check_fd(cp, notimeout))
 		{
-			if (pool_check_fd(cp, 0))
+			if (cp->issecondary_backend)
 			{
 				pool_log("pool_read: secondary data is not ready. abort this session");
 				exit(1);
 			}
+			else
+			{
+				pool_error("pool_read: pool_check_fd failed (%s)", strerror(errno));
+			    return -1;
+			}
 		}
 
 		readlen = read(cp->fd, readbuf, READBUFSZ);
@@ -187,6 +196,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 	int alloc_size;
 	int consume_size;
 	int readlen;
+	int notimeout;
 
 	req_size = cp->len + len;
 
@@ -208,15 +218,27 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 	len -= consume_size;
 	buf += consume_size;
 
+	/* if this is the secondary backend, then set timeout */
+	notimeout = !cp->issecondary_backend;
+#ifdef NOT_USED
+	pool_debug("notimeout: %d isbackend: %d issecondary_backend: %d", notimeout,
+			   cp->isbackend, cp->issecondary_backend);
+#endif
+
 	while (len > 0)
 	{
-		if (cp->issecondary_backend)
+		if (pool_check_fd(cp, notimeout))
 		{
-			if (pool_check_fd(cp, 0))
+			if (cp->issecondary_backend)
 			{
 				pool_log("pool_read2: secondary data is not ready. abort this session");
 				exit(1);
 			}
+			else
+			{
+				pool_error("pool_read2: pool_check_fd failed (%s)", strerror(errno));
+			    return NULL;
+			}
 		}
 
 		readlen = read(cp->fd, buf, len);
@@ -317,6 +339,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 	int strlength;
 	int flag;
 	int consume_size;
+	int notimeout;
 
 #ifdef DEBUG
 	static char pbuf[READBUFSZ];
@@ -383,9 +406,25 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 		readsize = cp->sbufsz;
 	}
 
+	/* if this is the secondary backend, then set timeout */
+	notimeout = !cp->issecondary_backend;
 
 	for (;;)
 	{
+		if (pool_check_fd(cp, notimeout))
+		{
+			if (cp->issecondary_backend)
+			{
+				pool_log("pool_read_string: secondary data is not ready. abort this session");
+				exit(1);
+			}
+			else
+			{
+				pool_error("pool_read_string: pool_check_fd failed (%s)", strerror(errno));
+			    return NULL;
+			}
+		}
+
 		readlen = read(cp->fd, cp->sbuf+readp, readsize);
 		if (readlen == -1)
 		{
